#ifndef _ADS_ERROR_H_
#define _ADS_ERROR_H_


#define ADS_ERROR								(1 << 29)	// Set bit 29 (ADS defined)

#define ADS_SEVERITY_SUCCESS 					(0 << 30)
#define ADS_SEVERITY_INFORMATIONAL				(1 << 30)
#define ADS_SEVERITY_WARNING					(2 << 30)
#define ADS_SEVERITY_ERROR 						(3 << 30)


// Facility Code Definitions
//--------------------------
#define FACILITY_DIO							(0x10 << 16)
#define FACILITY_CAN							(0x11 << 16)
#define FACILITY_I2C							(0x12 << 16)
#define FACILITY_FCD							(0x13 << 16)
#define FACILITY_MDI							(0x14 << 16)
#define FACILITY_PWR							(0x15 << 16)
#define FACILITY_USB							(0x16 << 16)

// Generic Error Code Definitions
//-------------------------------
// General Error Codes
#define ADS_GENERAL_ERROR						(0 | ADS_ERROR | ADS_SEVERITY_ERROR)
#define ADS_UNSPECIFIED_ERROR					(ADS_GENERAL_ERROR+0)
#define ADS_NO_ERROR							(ADS_GENERAL_ERROR+1)
#define ADS_OUTPUT_BUFFER_TOO_SMALL				(ADS_GENERAL_ERROR+2)
#define ADS_INPUT_BUFFER_WRONG_SIZE				(ADS_GENERAL_ERROR+3)
#define ADS_NOT_IMPLEMENTED						(ADS_GENERAL_ERROR+4)
#define ADS_INVALID_INPUT						(ADS_GENERAL_ERROR+5)
#define ADS_TIMEOUT_EXPIRED						(ADS_GENERAL_ERROR+6)
#define ADS_INPUT_OUT_OF_RANGE					(ADS_GENERAL_ERROR+7)

// File I/O Errors (also applicable to device I/O)
#define ADS_FILE_IO_ERROR						( 0x100 | ADS_ERROR | SEVERITY_ERROR )
#define	ADS_FILE_NOT_OPEN						( ADS_FILE_IO_ERROR+0 )
#define ADS_NO_READ_PERMISSION					( ADS_FILE_IO_ERROR+1 )
#define ADS_NO_WRITE_PERMISSION					( ADS_FILE_IO_ERROR+2 )
#define ADS_INVALID_HANDLE						( ADS_FILE_IO_ERROR+3 )
#define ADS_CANNOT_OPEN_DEVICE					( ADS_FILE_IO_ERROR+4 )
#define ADS_FILE_DOES_NOT_EXIST					( ADS_FILE_IO_ERROR+5 )
#define ADS_CANNOT_READ_FILE					( ADS_FILE_IO_ERROR+6 )


// Memory Errors
#define ADS_MEMORY_ERROR						(0x300 | ADS_ERROR | SEVERITY_ERROR)
#define ADS_CANNOT_ALLOC_MEMORY					(ADS_MEMORY_ERROR + 0)

// Communication Errors
#define ADS_COMM_ERROR							(0x400 | ADS_ERROR | SEVERITY_ERROR)
#define ADS_BUS_OFF								(ADS_COMM_ERROR+0)
#define ADS_BUS_ERROR							(ADS_COMM_ERROR+1)
#define ADS_NACK_RECEIVED						(ADS_COMM_ERROR+2)
#define ADS_PROTOCOL_ERROR						(ADS_COMM_ERROR+3)


// Driver API Usage Errors
#define ADS_API_ERROR							(0x500 | ADS_ERROR | SEVERITY_ERROR)
#define ADS_INVALID_IOCTL_CODE					(ADS_API_ERROR+0)


// Device Specific Error Code Definitions
//---------------------------------------

// Digital I/O Error Codes
#define DIO_UNSPECIFIED_ERROR				(ADS_UNSPECIFIED_ERROR | FACILITY_DIO)
#define DIO_NO_ERROR						(ADS_NO_ERROR | FACILITY_DIO)
#define DIO_ERROR_FILE_NOT_OPEN				(ADS_FILE_NOT_OPEN | FACILITY_DIO)
#define DIO_ERROR_NO_READ_PERMISSION		(ADS_NO_READ_PERMISSION | FACILITY_DIO)
#define DIO_ERROR_NO_WRITE_PERMISSION		(ADS_NO_WRITE_PERMISSION | FACILITY_DIO)
#define	DIO_ERROR_OUTPUT_BUFFER_TOO_SMALL	(ADS_OUTPUT_BUFFER_TOO_SMALL | FACILITY_DIO)
#define DIO_ERROR_INPUT_BUFFER_WRONG_SIZE	(ADS_INPUT_BUFFER_WRONG_SIZE | FACILITY_DIO)
#define DIO_ERROR_NOT_IMPLEMENTED			(ADS_NOT_IMPLEMENTED | FACILITY_DIO)
#define DIO_ERROR_INVALID_INPUT				(ADS_INVALID_INPUT | FACILITY_DIO)
#define DIO_ERROR_INVALID_IOCTL_CODE		(ADS_INVALID_IOCTL_CODE | FACILITY_DIO)

// CAN Error Codes
#define	CAN_ERROR_OUTPUT_BUFFER_TOO_SMALL	(ADS_OUTPUT_BUFFER_TOO_SMALL | FACILITY_CAN)
#define CAN_ERROR_INPUT_BUFFER_WRONG_SIZE	(ADS_INPUT_BUFFER_WRONG_SIZE | FACILITY_CAN)
#define CAN_ERROR_INVALID_HANDLE			(ADS_INVALID_HANDLE | FACILITY_CAN)
#define CAN_ERROR_CANNOT_OPEN_DEVICE		(ADS_CANNOT_OPEN_DEVICE | FACILITY_CAN)
#define CAN_ERROR_CANNOT_ALLOC_MEMORY		(ADS_CANNOT_ALLOC_MEMORY | FACILITY_CAN)
#define CAN_ERROR_BUS_OFF					(ADS_BUS_OFF | FACILITY_CAN)
#define CAN_ERROR_INPUT_OUT_OF_RANGE		(ADS_INPUT_OUT_OF_RANGE | FACILITY_CAN)

// I2C Error Codes
#define I2C_OPERATION_TIMEOUT				(ADS_TIMEOUT_EXPIRED | FACILITY_I2C)
#define I2C_ERROR_BUS_ERROR					(ADS_BUS_ERROR| FACILITY_I2C)
#define I2C_ERROR_INVALID_IOCTL_CODE		(ADS_INVALID_IOCTL_CODE | FACILITY_I2C)
#define I2C_ERROR_OUTPUT_BUFFER_TOO_SMALL	(ADS_OUTPUT_BUFFER_TOO_SMALL | FACILITY_I2C)
#define I2C_ERROR_NACK_RECEIVED				(ADS_NACK_RECEIVED | FACILITY_I2C)

// FCD Error Codes
#define FCD_ERROR_INVALID_IOCTL_CODE		(ADS_INVALID_IOCTL_CODE | FACILITY_FCD)
#define FCD_ERROR_FILE_DOES_NOT_EXIST		(ADS_FILE_DOES_NOT_EXIST | FACILITY_FCD)
#define FCD_ERROR_PROTOCOL_ERROR 			(ADS_PROTOCOL_ERROR | FACILITY_FCD)
#define FCD_ERROR_CANNOT_READ_FILE			(ADS_CANNOT_READ_FILE | FACILITY_FCD)
#define FCD_ERROR_OUTPUT_BUFFER_TOO_SMALL	(ADS_OUTPUT_BUFFER_TOO_SMALL | FACILITY_FCD)
#define FCD_ERROR_CANNOT_ALLOC_MEMORY 		(ADS_CANNOT_ALLOC_MEMORY | FACILITY_FCD)

// MDI Error Codes
#define MII_ERROR_INVALID_IOCTL_CODE		(ADS_INVALID_IOCTL_CODE | FACILITY_MDI)
#define MII_ERROR_FILE_DOES_NOT_EXIST		(ADS_FILE_DOES_NOT_EXIST | FACILITY_MDI)
#define MII_ERROR_PROTOCOL_ERROR 			(ADS_PROTOCOL_ERROR | FACILITY_MDI)
#define MII_ERROR_CANNOT_READ_FILE			(ADS_CANNOT_READ_FILE | FACILITY_MDI)
#define MII_ERROR_OUTPUT_BUFFER_TOO_SMALL	(ADS_OUTPUT_BUFFER_TOO_SMALL | FACILITY_MDI)

//POWER Error Codes
#define PWR_ERROR_OUTPUT_BUFFER_TOO_SMALL	(ADS_OUTPUT_BUFFER_TOO_SMALL | FACILITY_PWR)

//USB HCD Error Codes
#define HCD_ERROR_INVALID_IOCTL_CODE		(ADS_INVALID_IOCTL_CODE | FACILITY_USB)

#endif //_ADS_ERROR_H_
